/* ============================================================================
 * File Name: SLR10_Core_v0_0.v
 * Version 0.0 (build 02d)
 * 
 * Description:
 *   SLR10: 10-bit digital Slew Rate controller component (v 0.0)
 *   Produces ramp with controlled slope for input clock
 *
 * Credits:
 *
 * Note:
 *   frequency resolution = clock_frequency / 2^10
 *   frequency maximum    = clock_frequency / 2
 *   frequency minimum    = clock_frequency / 2^10
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/

//`#start header` -- edit after this line, do not edit this line
`include "cypress.v"
`ifdef SLR10_Core_v0_0_V_ALREADY_INCLUDED
`else
`define SLR10_Core_v0_0_V_ALREADY_INCLUDED
//`#end` -- edit above this line, do not edit this line
// Generated on 10/29/2020 at 22:13
// Component: SLR10_Core_v0_0

module SLR10_Core_v0_0( load, en, modul, clock, res, outp, rdy, rfd, co, mout );//

  // outp  parameters //
  
  parameter   outp_width = 8'd8;    // outp bus width (variable)
  localparam  Hi1 = outp_width-1;   // outp high bit index

  // Inputs & outputs //

  input  wire           load;	    // load data
  input  wire           en;         // DDS enable  
  input  wire           modul;      // external modulation: increment (1) / decrement (0)  
  input  wire           clock;      // input clock
  input  wire           res;        // reset  
  
  output wire [Hi1:0]   outp;       // outp bus
  output wire           rdy;        // outp data ready
  output wire           rfd;        // ready for new data (phase or shift) from API
  output wire           co;         // carry over
  output wire           mout;       // modulation output (modul input synched with clk)
  
  
  
  // Parameters // 
   
  parameter hw_load        = 1'b0;   // strobe on load (1) API update (0)
  parameter tune_word_h    = 9'd1;   // hardcoded tune word 10-bit 
  
  localparam CTRL_API      = 1'b0;   // API call
  localparam CTRL_HARDC    = 1'b1;   // Hardcoded
  parameter  ControlFreq   = CTRL_API; // control frequency by: API csll | Hardcoded | Digital bus
  
  localparam ENMODE_AUTO   = 2'b00;
  localparam ENMODE_CRONLY = 2'b01;
  localparam ENMODE_HWONLY = 2'b10;
  localparam ENMODE_CR_HW  = 2'b11;
  parameter [1:0] EnableMode = ENMODE_CRONLY; // auto enable | software only | hardware only | HW and SW 
  
  
  
  // no connect wires //
  
  wire [8:0] tune_word;         // tune word (10-bit)
  
  wire       ClockOutFromEnBlock;  // main clock for DDS core - 
  wire       s_modul;                // modul synched with ClockOutFromEnBlock


//============================================================================== 
//                             DDS Accumulator
//
//          produce single or dual outputs with phase shift
//============================================================================== 

    reg  [ 9:0]   Acc;              // 10-bit accumulator
    reg  s_modul_prev;
   
    
    // 10-bit->
    reg  co_r;      // overload byte 
    reg  co_r_prev; // overload byte previous 
    assign co = (co_r | co_r_prev);
    //assign co = co_r;
    
    always @(posedge ClockOutFromEnBlock) s_modul_prev <= s_modul;
    
    always @(posedge ClockOutFromEnBlock) begin	// sync
        if (s_modul != s_modul_prev) begin      // edge            
            {co_r, Acc} <= 0;                   // reset co_r and Acc 
             co_r_prev  <= 0;                   // reset
        end    
        else begin                              // s_modul==s_modul_prev            
            {co_r, Acc} <= Acc + tune_word;	    // increment // 30:46:58.069 MHz
             co_r_prev  <= co_r_prev | co_r;    // latch co_r flip state
        end
    end
        
    ///wire [Hi1:0] tmp_w = (co_r | co_r_prev)? 8'hFF : Acc[9:(9-Hi1)];      // 
    wire [Hi1:0] tmp_w = (co)? 8'hFF : Acc[9:(9-Hi1)];      // 29:46:58.021 MHz (in-phase)
    assign outp = (s_modul_prev)? tmp_w : ~tmp_w; // output  

    ///wire [Hi1:0] tmp_w = (co_r | co_r_prev)? 0 : ~Acc[9:(9-Hi1)];      // 
    //wire [Hi1:0] tmp_w = (co)? 0 : ~Acc[9:(9-Hi1)];      // 29:46:58.075 MHz (out-of-phase)
    //assign outp = (s_modul_prev)? tmp_w : ~tmp_w; // output  
       
     
       
//==============================================================================
//                              Count direction   
//  
//  count up when modul=1 , count down whem modul = 0
//==============================================================================
   
    assign mout = s_modul;
    //assign mout = s_modul_prev;
    
    //this realization consumes 1 status cell and makes 1.5-2 clock delay
    cy_psoc3_sync ModSync (                 // consumes 1 status cell to make DFF
            //.clock	(ClockOutFromEnBlock),  // output clock
            .clock	(clock),                // input clock
        	.sc_in	(modul),                  //  
	    	.sc_out	(s_modul));               //  
    
    //assign s_modul = modul; // no sync                     
         
//============================================================================== 
//              Data ready: sync outputs for bus outp and out2  
//
//  produce a single pulse of high polarity 1/2 clock length and 1/2 clock delay
//============================================================================== 

    reg [Hi1:0] outp_old;   //outp previous value
    
        always @(posedge ClockOutFromEnBlock) 	    
        begin
           outp_old <= outp;
        end
        assign rdy = |(outp ^ outp_old) & ~clock;  //pos edge:  delay 1/2 clock, width 1/2 clock        
    

//==============================================================================
//    Enable control (Auto, Software, Hardware, Software & Hardware)  
//
//  Uses UDB_clock_enable to enable DDS output (ClockOutFromEnBlock = clock & en)
//  Uses control register (CR) for API control when Enable Mode = SW or SW_HW
//==============================================================================
    
    
    wire clock_en;                                          // temp 
    
    cy_psoc3_udb_clock_enable_v1_0 #(.sync_mode(`TRUE))
    //cy_psoc3_udb_clock_enable_v1_0 #(.sync_mode(`FALSE))
    clock_enable_block (
                        .clock_out(ClockOutFromEnBlock),    // output clock
                        .clock_in(clock),                   // input clock
                        .enable(clock_en)                   // input enable     
    );
    
                                                     
    localparam CLOCK_ENABLE_BIT   = 1'd0;                   // reserve bit 0 for clock enable                                                
    wire [7:0] ctrl8;                                       // Control Register bit location (bits 7-1 are unused)
    
    generate
    if( (EnableMode == ENMODE_CRONLY) || (EnableMode == ENMODE_CR_HW) ) begin : sCTRLReg 
        cy_psoc3_control #(.cy_force_order(`TRUE))          //Default mode
            ctrlreg( .control(ctrl8) );
    end
    endgenerate
    
    wire control_enable; assign control_enable = ctrl8[CLOCK_ENABLE_BIT];

    assign clock_en = (EnableMode == ENMODE_AUTO)?   1'b1:                  // auto enable
                      (EnableMode == ENMODE_CRONLY)? control_enable:        // software_only  
                      (EnableMode == ENMODE_CR_HW)? (control_enable & en):  // software_and_hardware
                                                     en;                    // hardware_only
    
    

//==============================================================================
//                              Ready for data   
//  
//  goes high when acomponent is ready to update control phase or delay registers
//==============================================================================
   
    
    //this realization consumes 1 status cell and makes 1.5-2 clock delay
    generate                                    // resource does not removed automatically
    if (hw_load==1'b1) begin                    // hardware load = true, rfd is visible
        wire s_load;                            // synched load
        cy_psoc3_sync HWLoadSync (              // consumes 1 status cell to make DFF
                .clock	(ClockOutFromEnBlock),
    			.sc_in	(load),                 //  
	    		.sc_out	(s_load));              //  
                         
        assign rfd = s_load;                    // delay 1 full clock
    end
    endgenerate

    
    //this relaizes instant alternative way with no delays, but async output
    //assign rfd = clock_en && load;            // instant async update 
                                    
                                   
                                    
//==============================================================================
//              DDS control source: API / Hardcoded / HW bus
//
// use API, Hardcoded value or Digital bus to control DDS frequency and phase
//==============================================================================
 
    wire [23:0]  control_cr;                    // Control Register output phase increment

    // 3:1 frequency mux: API | Hardcoded | Digital bus
    generate  
    if (ControlFreq==CTRL_API)   assign tune_word = control_cr[8:0];  else // API 10-bit
    if (ControlFreq==CTRL_HARDC) assign tune_word = tune_word_h;            // Hardcoded 
    endgenerate 
       

//==============================================================================
//          API control registers to set phase increment and shift
//
//  tune_word = {ControlPhaseC, ControlPhaseB, ControlPhaseA}
//  where ControlPhaseA, ControlPhaseB, ControlPhaseC - are Lo, Me, Hi bits
//
//  options: (i) direct API update or (ii) buffered with hardware strobe
//==============================================================================


    generate                                
    if (hw_load==1'b0) begin: sControl                                          // direct API update
 
            cy_psoc3_control #(.cy_init_value (8'h00), .cy_force_order(`TRUE))  // default mode
            ControlPhaseA(
                .control(control_cr[7:0]));                                     // output Lo bits [07:00]

            cy_psoc3_control #(.cy_init_value (8'h00), .cy_force_order(`TRUE))  // default mode
            ControlPhaseB(
                .control(control_cr[15:8]));                                    // output Md bits [15:08]
    
    end
    else begin: sControl                                                        // buffered update on hardware load 

            cy_psoc3_control #(.cy_init_value (8'h00), .cy_force_order(`TRUE), .cy_ctrl_mode_1(8'h00), .cy_ctrl_mode_0(8'hFF)) //Sync mode
            ControlPhaseA(
                .control(control_cr[7:0]),                                      // output Lo bits [07:00]
                .clock(load)            );                                      // hw load 
            
            cy_psoc3_control #(.cy_init_value (8'h00), .cy_force_order(`TRUE), .cy_ctrl_mode_1(8'h00), .cy_ctrl_mode_0(8'hFF)) //Sync mode
            ControlPhaseB(
                .control(control_cr[15:8]),                                     // output Md bits [15:08]
                .clock(load)             );                                     // hw load 
    
    end
    endgenerate
   

//==============================================================================




endmodule
`endif /* SLR10_Core_v0_0_V_ALREADY_INCLUDED */
////////////////////////////////////////////////////////////////////////////////
//`#end` -- edit above this line, do not edit this line
//`#start footer` -- edit after this line, do not edit this line
//`#end` -- edit above this line, do not edit this line
