/* ============================================================================
 * File Name: SLR10.* v0_0
 * 
 * Description:
 *   SLR10: 10-bit slew rate generator component (v0.0)
 *   realized entirely in PLD
 *
 * Note:
 *   based on DDS24 code:
 *   https://community.cypress.com/thread/13628 
 *
 * Note:
 *   frequency resolution = clock_frequency / 2^10 
 *   frequency maximum    = clock_frequency / 2
 *   frequency minimum    = clock_frequency / 2^10
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#ifndef `$INSTANCE_NAME`_H
#define `$INSTANCE_NAME`_H

#include <project.h>


#define true  1 // but not True
#define false 0 // but not False
    
    
/***************************************
*   Conditional Compilation Parameters
***************************************/

#define `$INSTANCE_NAME`_CLOCK_FREQ (double) `=$ClockFreq                 `   // input clock frequency
#define `$INSTANCE_NAME`_HARDWARE_LOAD       `=$HardwareLoad              `   // load frequency and phase on hardware digital signal 
#define `$INSTANCE_NAME`_CR_ENABLE           `=($EnableMode == CR_only || $EnableMode == HW_and_CR)`  // enable mode = SW or SW_&_HW
#define `$INSTANCE_NAME`_CONTROL_FREQ_API    `=($ControlFreq  == Ctrl_API)`   // use API to set DDS frequency
#define `$INSTANCE_NAME`_CONTROL_PHASE_API   `=($ControlPhase == Ctrl_API)`   // use API to set out2 phase
 
    
/***************************************
*           API Constants
***************************************/   

#define `$INSTANCE_NAME`_PresetFreq          `=$Frequency `  // preset startup frequency



/***************************************
*           properties
***************************************/   

//CYBIT  `$INSTANCE_NAME`_Initialized;  // 
//CYBIT  `$INSTANCE_NAME`_Enabled;      //
    
/***************************************
*        Function Prototypes
***************************************/
    
void   `$INSTANCE_NAME`_Init();                      // initialize DDS
void   `$INSTANCE_NAME`_Start();                     // Init and Enable DDS
void   `$INSTANCE_NAME`_Stop();                      // Stop (disable) DDS 
void   `$INSTANCE_NAME`_Enable();                    // Enable DDS (must init first) // todo: add _Enabled and _Initialized property
uint8  `$INSTANCE_NAME`_SetFrequency( double Freq ); // Set desired output frequency
double `$INSTANCE_NAME`_GetOutFreq();                // read actual output frequency
double `$INSTANCE_NAME`_GetFrequency();              // return current set frequency 


#endif // `$INSTANCE_NAME`_H


/* [] END OF FILE */
